// icalc.h -- Larry Medwin 4/5/93
// InterViews user interface for the constraint system

#ifndef __ICALC_H__
#define __ICALC_H__

#include <IV-look/dialogs.h>
#include <IV-look/field.h>
#include <IV-look/kit.h>
#include <InterViews/background.h>
#include <InterViews/composition.h>
#include <InterViews/deck.h>
#include <InterViews/patch.h>
#include <InterViews/layout.h>
#include <InterViews/resource.h>
#include <InterViews/session.h>
#include <InterViews/simplecomp.h>
#include <InterViews/style.h>
#include <InterViews/window.h>
#include <OS/string.h>
#include <stream.h>
#include <stdio.h>

#include "constraints.h"

extern class IVSym;
extern class Lexio;
extern class App;

extern Style* global_style;
extern App* Myapp;

declareFieldEditorCallback(App)
declareFieldEditorCallback(IVSym)
declareActionCallback(IVSym)

extern int yyparse();

class IVSym : public Sym , public MonoGlyph {
  public:
    IVSym(Sym*);
    ~IVSym() {};

    // FieldEditor callbacks
    void accept_editor(FieldEditor*);
    void cancel_editor(FieldEditor*);

    // Pushbutton callback
    void click_me();

    static Style* sym_style;    // set by App::App()

  private:
    void IVSym_init();

    // UI stuff
    FieldEditor* ed_;           // User enters values here
    FieldEditor* uned_;         // Replaces ed_ when not forced
    Deck* deck_;
    Patch* patch_;
    Button* pbutton_;           // Displays name_

    // Display value and state
    virtual void show_val();
    virtual void show_no_val();
    virtual void show_state();
};

// Replace lex I/O handlers to read from char arrays instead of stdio
class Lexio {
  private:
    char* equation_;            // char string for lex
    char* current_;             // cursor for lex
  public:
    Lexio(String);
    ~Lexio();

    // Undefine lex I/O macros
#   undef lex_input()
#   undef unput(char)

    // Lex I/O handlers
    int lex_input();
    void unput(char);
};

// Canvas with equation editor and ActiveSyms
class App {
  private:
    String equation_;       // FieldEditor String
    IVSym* network_;        // Access to constraint network
  public:
    App(Style*);
    Lexio* lex_handler;

    // InterViews components
    FieldEditor* eq_editor_;
    Glyph* sym_box_;        // vbox to hold sym_area_

    // FieldEditor callbacks
    void accept_editor(FieldEditor*);
    void cancel_editor(FieldEditor*);

    // Start of App with initial equation
    void kick_start(char *);        // Shouldn't need this
};


#endif // __ICALC_H__
