%{
/*
 * lex.l
 *   lexical analysis for constraint calculator
 *   borrowed heavily from Schreiner and Friedman
 *
 */

#include "constraints.h"
#include "y.tab.h"
#include <string.h>

/* #define USING_INTERVIEWS in Makefile to use the InterViews user interface */
#define USING_INTERVIEWS
#ifdef USING_INTERVIEWS
#include "icalc.h"

// Redefine Lex I/O handlers
int lex_input() { return Myapp->lex_handler->lex_input(); };
void unput(char c) { Myapp->lex_handler->unput(c); };
#endif /* USING_INTERVIEWS */

static int screen();
%}

letter              [a-zA-Z_]
digit               [0-9]
letter_or_digit     [a-zA-Z_0-9]
white_space         [ \t\n]
blank               [ \t]
other               .

%%

{letter}{letter_or_digit}*  return screen();

{digit}+            { yylval.y_float = atof(yytext);
                      return CONSTANT;
                    }

{white_space}+      ;


{other}             return yytext[0];

%%

/*
 *  Reserved word scanner
 */

static int screen()
{
    if (strncmp( "set", yytext, strlen("set")) == 0)
        return SET;
    if (strncmp( "unset", yytext, strlen("unset")) == 0)
        return UNSET;
    if (strncmp( "equation", yytext, strlen("equation")) == 0) {
        delete Symtab;
        Symtab = new SymbolList;
        Phase = PARSE;
        return EQUATION;
    }

    /* Must be a Sym */

    if (Phase == PARSE) {
        /* Lookup or insert it; then return it on value stack */
        yylval.y_sym = Symtab->sym_lookup(yytext);

        return VARIABLE;
    }

    /* Phase == COMMAND */
    else if (Phase == COMMAND) {
        yylval.y_sym = Symtab->sym_find(yytext);

        return VARIABLE;
    }

    /* Phase == INIT */
    else {
        cout << "Type 'equation' before entering equation\n";
        cout.flush();
    }
}
