//--------------------------------------------------------------
//  PT.CPP - Periodic Table for Windows
//--------------------------------------------------------------
#include <afxwin.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

//#include <medw.h>		// this won't work with SC++ 6.0

#include "resource.h"
#include "pt.h"
#include "ptatom.h"

BOOL CPTDialog::bRegistered = FALSE;
HACCEL hAccelerator;

//--------------------------------------------------------------
// CPTDialog Constructor
//      Registers our class if necessary, constructs the
//      dialog object, and populates the combo boxes.
//--------------------------------------------------------------
CPTDialog::CPTDialog()
{
    int i;

    if (!bRegistered)
        bRegistered = RegisterPTClass(); 

    Create ("pt");
        
    for (i=1; i<=PT_MAXELEMENTS; ++i)
    {
        CWnd::SendDlgItemMessage (IDD_ELEMENTNAME, CB_ADDSTRING,
                            0, (LONG)(LPSTR)_atom[i].element);
        CWnd::SendDlgItemMessage (IDD_ELEMENTSYMBOL, CB_ADDSTRING,
                            0, (LONG)(LPSTR)_atom[i].symbol);
    }
    // No combo-box has focus.
    memset (CB_Focus, 0, sizeof (CB_Focus));
}
//--------------------------------------------------------------
//  RegisterPTClass
//      Register the PT dialog class and replace the default
//      white box icon with our own.
//
//  Returns: nonzero if class is registered.
//           zero if class is not registered.
//--------------------------------------------------------------
BOOL CPTDialog::RegisterPTClass()
{
    WNDCLASS wndclass;

    wndclass.style = 0;
    wndclass.lpfnWndProc = DefDlgProc;  /* use default dialog proc */
    wndclass.cbClsExtra = 0;

    // This field MUST be set to DLGWINDOWEXTRA, or this class we're
    // registering won't work properly with our dialog boxes.

    // MUST be set to DLGWINDOWEXTRA or class won't work properly
    wndclass.cbWndExtra = DLGWINDOWEXTRA ;

    // Use MFC global function to retrieve app's instance.
    wndclass.hInstance = AfxGetInstanceHandle();

    // Load custom icon, cursor, set background, load menu
    wndclass.hIcon = ::LoadIcon (AfxGetInstanceHandle(), "PTICON");
    wndclass.hCursor = ::LoadCursor (NULL, IDC_ARROW) ;
    wndclass.hbrBackground  = (HBRUSH) COLOR_WINDOW + 1;
    wndclass.lpszMenuName   = "PTMenu";

    // Need unique name here.  Name must be used in dialog box
    // template in DIALOGS.DLG to force dialog box to use this
    // class.
    wndclass.lpszClassName  = "PTDLGCLASS";
    
    //hAccelerator = ::LoadAccelerators (wndclass.hInstance, "AccelTable");
    return ::RegisterClass(&wndclass);
}
//--------------------------------------------------------------
//  GetPTData
//      Retrieve and display data based on location of
//      input focus: name, symbol, atomic number, or 
//      atomic weight.  Accepts control ID as input.
//--------------------------------------------------------------
void CPTDialog::GetPTData (int nID)
{
    char buffer[20];
    char *pWeight;
    double fBuffer;
    int i, bytes;

    bytes = CWnd::GetDlgItemText (nID, (LPSTR)buffer, sizeof(buffer));

    if (bytes > 0)
    {
        switch (nID)
        {
        case IDD_ELEMENTNAME:
            for (i=1; i<=PT_MAXELEMENTS; ++i)
            {
                if (stricmp (buffer, _atom[i].element) == 0)
                    break;
            }
            break;
        case IDD_ELEMENTSYMBOL:
            for (i=1; i<=PT_MAXELEMENTS; ++i)
            {
                if (stricmp (buffer, _atom[i].symbol) == 0)
                    break;
            }
            break;
        case IDD_ATOMICNUMBER:
            i = atoi (buffer);
            break;
        case IDD_ATOMICWEIGHT:
            //  Atomic weights stored as strings.  Convert
            //  to floating point to do comparisons.
            fBuffer = atof (buffer);
            for (i=1; i<=PT_MAXELEMENTS; ++i)
            {
                //  Some weights are in parens.
                //  Skip over them if detected.
                if (_atom[i].weight[0] != '(')
                    pWeight = _atom[i].weight;
                else
                    pWeight = &_atom[i].weight[1];

                if (atof (pWeight) >= fBuffer)
                    break;
            }
            break;
        default:
            i = 0;
            break;
        }
        //  i contains the index into the array of structures:
        //  _atom[i].xxxx
        if (i <= PT_MAXELEMENTS && i > 0)
            Display (i+100);
        else
            MessageBox ("Selected element not found in periodic table.",
                        "ERROR",
                        MB_OK | MB_ICONEXCLAMATION);
    }
}
//--------------------------------------------------------------
//  OnNameSetFocus
//  OnSymbolSetFocus
//  OnNumberSetFocus
//  OnWeightSetFocus
//      These functions keep track of which field has the
//      input focus.  Fields are mapped to array CB_Focus[].
//      If user clicks on any edit fields or type in any
//      data, blank out any existing data with Display (100).
//--------------------------------------------------------------
void CPTDialog::OnNameSetFocus()
{
    Display (100);
    memset (CB_Focus, 0, sizeof(CB_Focus));
    CB_Focus[0] = 1;
}
void CPTDialog::OnSymbolSetFocus()
{
    Display (100);
    memset (CB_Focus, 0, sizeof(CB_Focus));
    CB_Focus[1] = 1;
}
void CPTDialog::OnNumberSetFocus()
{
    Display (100);
    memset (CB_Focus, 0, sizeof(CB_Focus));
    CB_Focus[2] = 1;
}
void CPTDialog::OnWeightSetFocus()
{
    Display (100);
    memset (CB_Focus, 0, sizeof(CB_Focus));
    CB_Focus[3] = 1;
}
//--------------------------------------------------------------
//  OnCancel
//      User pressed the "Cancel" button.  Terminate program.
//--------------------------------------------------------------
void CPTDialog::OnCancel()
{
    DestroyWindow();
}
//--------------------------------------------------------------
//  OnShowAtom
//      Draw selected atom in another dialog box.
//--------------------------------------------------------------
void CPTDialog::OnShowAtom ()
{
	char szAtomName [12+1];

	if (GetDlgItemText (IDD_ELEMENTNAME, szAtomName, sizeof (szAtomName)) == 0)
		MessageBox ("Select an element first.", "Error", MB_OK|MB_TASKMODAL|MB_ICONINFORMATION);
	else	
		CATOMDialog *ATOM = new CATOMDialog (szAtomName);
}

//--------------------------------------------------------------
//  OnOK
//      User pressed the "OK" button.  Retrieve element info
//      based on the edit box which has the input focus.
//--------------------------------------------------------------
void CPTDialog::OnOK()
{
    //  CB_Focus tracks which edit box user entered data.  GetPTData
    //  searches for that data.
    if (CB_Focus[0])
        GetPTData (IDD_ELEMENTNAME);
    else if (CB_Focus[1])
        GetPTData (IDD_ELEMENTSYMBOL);
    else if (CB_Focus[2])
        GetPTData (IDD_ATOMICNUMBER);
    else
        GetPTData (IDD_ATOMICWEIGHT);
}
//--------------------------------------------------------------
//  OnAbout
//      User selected "About" from menu.  Open a modal dialog
//      box and tell user about this program.
//--------------------------------------------------------------
void CPTDialog::OnAbout()
{
    CModalDialog about( "AboutBox", this );
    about.DoModal();
}
//--------------------------------------------------------------
//  OnHelp
//      User selected help from the menu.  Open a modal dialog
//      box and display help info.
//--------------------------------------------------------------
void CPTDialog::OnHelp()
{
    CModalDialog version ("HelpBox", this);
    version.DoModal();
}
//--------------------------------------------------------------
//  Display
//      Display element info in the edit boxes using
//      "SetDlgItemText"
//--------------------------------------------------------------
void CPTDialog::Display(int iAtomicNumber)
{
    char szNumber[4];

    //  Atomic numbers range from 1 to 108 but to avoid conflict 
    //  with IDOK and IDCANCEL (1 & 2), 100 has been added to
    //  the atomic numbers.  Adjust before indexing array.
    iAtomicNumber -= 100;

    //  Convert atomic number from numeric to string.
    if (iAtomicNumber <= 0)                        /* too small */
        memset (szNumber, 0, sizeof (szNumber));
    else if (iAtomicNumber > PT_MAXELEMENTS)       /* too big */
        sprintf (szNumber, "%3d", PT_MAXELEMENTS);
    else                                           /* juuust right */
        sprintf (szNumber, "%3d", _atom [iAtomicNumber].number);

    CWnd::SetDlgItemText (IDD_ELEMENTNAME, _atom [iAtomicNumber].element);
    CWnd::SetDlgItemText (IDD_ELEMENTSYMBOL, _atom [iAtomicNumber].symbol);
    CWnd::SetDlgItemText (IDD_ATOMICNUMBER, szNumber);
    CWnd::SetDlgItemText (IDD_ATOMICWEIGHT, _atom [iAtomicNumber].weight);
}
//--------------------------------------------------------------
//    MESSAGE MAP
//-------------------------------------------------------------- 
BEGIN_MESSAGE_MAP (CPTDialog, CDialog)
    //{{ AFX_MSG_MAP (CPTDialog)
    ON_WM_CLOSE ()
    ON_COMMAND (IDM_ABOUT, OnAbout)
    ON_COMMAND (IDM_HELP, OnHelp)
    ON_COMMAND (IDCANCEL, OnCancel)
    ON_COMMAND (IDOK, OnOK)	

    ON_COMMAND (IDD_SHOWATOM, OnShowAtom)
    
    ON_CBN_SETFOCUS  (IDD_ELEMENTNAME, OnNameSetFocus)
    ON_CBN_SETFOCUS  (IDD_ELEMENTSYMBOL, OnSymbolSetFocus)
    ON_EN_SETFOCUS  (IDD_ATOMICNUMBER, OnNumberSetFocus)
    ON_EN_SETFOCUS  (IDD_ATOMICWEIGHT, OnWeightSetFocus)
    
    ON_COMMAND (H, OnH)
    ON_COMMAND (He, OnHe)
    ON_COMMAND (Li, OnLi)
    ON_COMMAND (Be, OnBe)
    ON_COMMAND (B, OnB)
    ON_COMMAND (C, OnC)
    ON_COMMAND (N, OnN)
    ON_COMMAND (O, OnO)
    ON_COMMAND (F, OnF)
    ON_COMMAND (Ne,OnNe)
    ON_COMMAND (Na, OnNa)
    ON_COMMAND (Mg, OnMg)
    ON_COMMAND (Al, OnAl)
    ON_COMMAND (Si, OnSi)
    ON_COMMAND (P, OnP)
    ON_COMMAND (S, OnS)
    ON_COMMAND (Cl, OnCl)
    ON_COMMAND (Ar, OnAr)
    ON_COMMAND (K, OnK)
    ON_COMMAND (Ca, OnCa)
    ON_COMMAND (Sc, OnSc)
    ON_COMMAND (Ti, OnTi)
    ON_COMMAND (V, OnV)
    ON_COMMAND (Cr, OnCr)
    ON_COMMAND (Mn, OnMn)
    ON_COMMAND (Fe, OnFe)
    ON_COMMAND (Co, OnCo)
    ON_COMMAND (Ni, OnNi)
    ON_COMMAND (Cu, OnCu)
    ON_COMMAND (Zn, OnZn)
    ON_COMMAND (Ga, OnGa)
    ON_COMMAND (Ge, OnGe)
    ON_COMMAND (As, OnAs)
    ON_COMMAND (Se, OnSe)
    ON_COMMAND (Br, OnBr)
    ON_COMMAND (Kr, OnKr)
    ON_COMMAND (Rb, OnRb)
    ON_COMMAND (Sr, OnSr)
    ON_COMMAND (Y, OnY)
    ON_COMMAND (Zr, OnZr)
    ON_COMMAND (Nb, OnNb)
    ON_COMMAND (Mo, OnMo)
    ON_COMMAND (Tc, OnTc)
    ON_COMMAND (Ru, OnRu)
    ON_COMMAND (Rh, OnRh)
    ON_COMMAND (Pd, OnPd)
    ON_COMMAND (Ag, OnAg)
    ON_COMMAND (Cd, OnCd)
    ON_COMMAND (In, OnIn)
    ON_COMMAND (Sn, OnSn)
    ON_COMMAND (Sb, OnSb)
    ON_COMMAND (Te, OnTe)
    ON_COMMAND (I, OnI)
    ON_COMMAND (Xe, OnXe)
    ON_COMMAND (Cs, OnCs)
    ON_COMMAND (Ba, OnBa)
    ON_COMMAND (La, OnLa)
    ON_COMMAND (Ce, OnCe)
    ON_COMMAND (Pr, OnPr)
    ON_COMMAND (Nd, OnNd)
    ON_COMMAND (Pm, OnPm)
    ON_COMMAND (Sm, OnSm)
    ON_COMMAND (Eu, OnEu)
    ON_COMMAND (Gd, OnGd)
    ON_COMMAND (Tb, OnTb)
    ON_COMMAND (Dy, OnDy)
    ON_COMMAND (Ho, OnHo)
    ON_COMMAND (Er, OnEr)
    ON_COMMAND (Tm, OnTm)
    ON_COMMAND (Yb, OnYb)
    ON_COMMAND (Lu, OnLu)
    ON_COMMAND (Hf, OnHf)
    ON_COMMAND (Ta, OnTa)
    ON_COMMAND (W, OnW)
    ON_COMMAND (Re, OnRe)
    ON_COMMAND (Os, OnOs)
    ON_COMMAND (Ir, OnIr)
    ON_COMMAND (Pt, OnPt)
    ON_COMMAND (Au, OnAu)
    ON_COMMAND (Hg, OnHg)
    ON_COMMAND (Tl, OnTl)
    ON_COMMAND (Pb, OnPb)
    ON_COMMAND (Bi, OnBi)
    ON_COMMAND (Po, OnPo)
    ON_COMMAND (At, OnAt)
    ON_COMMAND (Rn, OnRn)
    ON_COMMAND (Fr, OnFr)
    ON_COMMAND (Ra, OnRa)
    ON_COMMAND (Ac, OnAc)
    ON_COMMAND (Th, OnTh)
    ON_COMMAND (Pa, OnPa)
    ON_COMMAND (U, OnU)
    ON_COMMAND (Np, OnNp)
    ON_COMMAND (Pu, OnPu)
    ON_COMMAND (Am, OnAm)
    ON_COMMAND (Cm, OnCm)
    ON_COMMAND (Bk, OnBk)
    ON_COMMAND (Cf, OnCf)
    ON_COMMAND (Es, OnEs)
    ON_COMMAND (Fm, OnFm)
    ON_COMMAND (Md, OnMd)
    ON_COMMAND (No, OnNo)
    ON_COMMAND (Lr, OnLr)
    ON_COMMAND (Unq, OnUnq)
    ON_COMMAND (Unp, OnUnp)
    ON_COMMAND (Unh, OnUnh)
    ON_COMMAND (Uns, OnUns)
    ON_COMMAND (Uno, OnUno)
    ON_COMMAND (Une, OnUne)
    //}} AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------
//  Create the application object
//--------------------------------------------------------------
CTheApp theApp; 
//--------------------------------------------------------------
//  InitInstance
//      Make CPTDialog object the main window by assigning
//      m_pMainWnd to constructor.
//
//  Returns:    TRUE
//--------------------------------------------------------------

BOOL CTheApp::InitInstance()
{
    m_pMainWnd = new CPTDialog();
    return TRUE;
}
//--------------------------------------------------------------
//                      End of PT.CPP
//--------------------------------------------------------------
