//--------------------------------------------------------------
//  PTATOM.CPP - Periodic Table for Windows
//--------------------------------------------------------------
#include <afxwin.h>
#include <windows.h>

#include <string.h>

#include "resource.h"
#include "ptatom.h"

//--------------------------------------------------------------
// CATOMDialog Constructor
//      Creates a modeless dialog box to display an atom.  Also
//		sets the dialog caption to the atom name and creates
//		the atom object.
//--------------------------------------------------------------
CATOMDialog::CATOMDialog(char *AtomName)
{
	if (stricmp (AtomName, "Hydrogen"))
	{
		MessageBox ("This version of PT can only draw Hydrogen",
					"SORRY",
					MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);
		return;
	}

	Atom = new CATOM (AtomName);
	
	if (Create ("ATOM") == FALSE)
		MessageBox ("Cannot create modeless dialog box.",
		 			"ERROR", MB_OK);
	else
		SetWindowText (AtomName);
}
//--------------------------------------------------------------
// ~CATOMDialog Destructor
//      Destroys the atom object and the modeless dialog box.
//--------------------------------------------------------------
CATOMDialog::~CATOMDialog()
{
	delete Atom;
	DestroyWindow();
}
//--------------------------------------------------------------
// OnOK
//		User pressed OK button.  Destroy the dialog box.
//--------------------------------------------------------------
void CATOMDialog::OnOK()
{
	delete this;
}
//--------------------------------------------------------------
// OnPaint
//		Received a WM_PAINT message. Get the device context and
//  	draw the atom inside the GroupBox.
//--------------------------------------------------------------
void CATOMDialog::OnPaint()
{
	CPaintDC pdc(this);		// paint device context on stack

	CWnd *GroupBox = GetDlgItem (IDD_GROUPBOX);
	if (GroupBox != NULL && Atom != NULL)
		Atom->DrawAtom (GroupBox);
}
//--------------------------------------------------------------
// CATOM Constructor
//--------------------------------------------------------------
CATOM::CATOM (char *szName)
{
	strcpy (Name, szName);
}
//--------------------------------------------------------------
// CATOM Destructor
//		No handling necessary.  Included for completeness.
//--------------------------------------------------------------
CATOM::~CATOM ()
{
}
//--------------------------------------------------------------
// DrawAtom
//		This method only draws the Hydrogen atom.  Output goes
//		to the CWnd object.  Drawing is done from the outside-in:
// 		electron shell, electron, then nucleus.
//--------------------------------------------------------------
int CATOM::DrawAtom(CWnd *Parent)
{
	RECT rc;
	int HorzSF, VertSF;		// scale factors

	CClientDC pdc(Parent);	// client device context on stack
	
	// scale down rectangle.  Use ellipse() to describe
	// electron orbits.around nucleus.
	Parent->GetClientRect (&rc);
	HorzSF = rc.right/5;
	VertSF = rc.bottom/5;
	
	rc.left = HorzSF;
	rc.top = VertSF;
	rc.right -= rc.left;
	rc.bottom -= rc.top;
	
	pdc.SelectStockObject (HOLLOW_BRUSH);
	pdc.SelectStockObject (BLACK_PEN);
	pdc.Ellipse (&rc);		// electron orbit

	// set up to draw electron.  Easier if we position
	// at 12 o'clock.	
	rc.left = rc.left + (rc.right - rc.left)/2 - 4;
	rc.top -= 4;
	rc.right = rc.left + 8;
	rc.bottom = rc.top + 8;
	
	pdc.SelectStockObject (BLACK_BRUSH);
	pdc.Ellipse (&rc);		// electron
	
	// scale down rectangle to draw nucleus.
	Parent->GetClientRect (&rc);
	
	rc.left = HorzSF*2;
	rc.top = VertSF*2; 
	rc.right -= rc.left;
	rc.bottom -= rc.top;
			
	pdc.SelectStockObject (BLACK_BRUSH);	
	pdc.Ellipse (&rc);		// nucleus
	
	return 0;
}
//--------------------------------------------------------------
//    MESSAGE MAP
//-------------------------------------------------------------- 
BEGIN_MESSAGE_MAP (CATOMDialog, CDialog)
    //{{ AFX_MSG_MAP (CPTATOMDialog)
    ON_WM_CLOSE ()
    ON_COMMAND (IDOK, OnOK)
    ON_WM_PAINT ()
    //}} AFX_MSG_MAP
END_MESSAGE_MAP()
